### MERGE Fushan DATA
rm(list = ls())
library(reshape, quietly=TRUE)
source("format.fun.R")
dir.create("../../output/formatted/Fushan", recursive=TRUE,showWarnings=FALSE)

######################### READ DATA read individuals tree data
load("../../data/raw/Fushan/fushan.rdata")
data.fushan <- data.frame(fushan)
rm(fushan)
### read species names
species.clean <- read.csv("../../data/raw/Fushan/Splist_Fushan_En.csv", stringsAsFactors = FALSE)


########################################## FORMAT INDIVIDUAL TREE DATA
data.fushan$year <- rep(5, nrow(data.fushan)) ## number of year between measurement
data.fushan$G <- 10*(data.fushan$dbh2 - data.fushan$dbh1)/data.fushan$year ## diameter growth in mm per year
data.fushan$D <- data.fushan[["dbh1"]] ## diameter in cm
data.fushan$dead <- as.numeric(data.fushan$status2 == "dead")  ## dummy variable for dead tree 0 alive 1 dead
data.fushan$plot <- rep(1,nrow(data.fushan))  ## plot code 
data.fushan$htot <- rep(NA,nrow(data.fushan)) ## height of tree in m
data.fushan$tree.id <- data.fushan$tag  ## tree unique id - use tag here as it is unique for each row
data.fushan$sp.name <- rep(NA,nrow(data.fushan)) 
v <- species.clean$sp
for (i in 1:length(unique(data.fushan$sp))) {
    sel.spp <- which(data.fushan$sp == unique(data.fushan$sp)[i])
    data.fushan$sp.name[sel.spp] <- paste(species.clean$family[sel.spp], species.clean$genus[sel.spp], 
        species.clean$epithet[sel.spp], sep = " ")
}
data.fushan$weights <- 1/(pi*(0.5*data.fushan$D/100)^2)
data.fushan$obs.id <- data.fushan$tag 
data.fushan$subplot <- data.fushan$q20
data.fushan$census <- rep(1,nrow(data.fushan)) 

data.fushan$x <- data.fushan$gx
data.fushan$y <- data.fushan$gy

####### PLOT SELECTION FOR THE ANALYSIS Remove data with dead == 1

### delete recruited tree
data.fushan <-  subset(data.fushan,subset=data.fushan$status1!="P")

vec.basic.var <- c("obs.id","tree.id", "sp", "sp.name","plot", "subplot", "D", "G", "dead", 
    "year", "htot", "x", "y", "census")
data.tree <- subset(data.fushan, select = c(vec.basic.var))
write.csv(data.tree,file="../../output/formatted/Fushan/tree.csv")



################ MASSAGE TRAIT DATA Obtain maximum height per species from data.trait no sd
################ available as we have only one observation for species
data.trait <- read.table("../../data/raw/Fushan/fs_trait_Kunstler.txt", header = T, 
    sep = "\t")
colnames(data.trait) <- c("sp", "sla", "wd", "seedmassmg", "meanN", "maxheightm")
data.trait <- merge(data.trait, data.frame(sp = species.clean$sp, Latin = apply(species.clean[,c("genus","epithet")],1,paste,collapse="_"), 
	stringsAsFactors = F), by = "sp")
data.trait$Leaf.N.mean <- data.trait$meanN*10; data.trait$meanN <- NULL; ## conversion from percentage to mg/g
data.trait$Leaf.N.sd <- NA
data.trait$Seed.mass.mean <- data.trait$seedmassmg; data.trait$seedmassmg <- NULL
data.trait$Seed.mass.sd <- NA
data.trait$SLA.mean <- data.trait$sla/10; data.trait$sla <- NULL
data.trait$SLA.sd <- NA
data.trait$Wood.density.mean <- data.trait$wd; data.trait$wd <- NULL
data.trait$Wood.density.sd <- NA
data.trait$Max.height.mean <- log10(data.trait$maxheightm); 
data.trait$Max.height.sd <- NA
data.max.height <- data.frame(sp = data.trait$sp, Max.height = log10(data.trait$maxheightm))
data.fushan <- merge(data.fushan, data.max.height, by = "sp")
data.trait$maxheightm <- NULL


###### SAVE TRAITS IN STD FORMAT
write.csv(data.trait,file='../../output/formatted/Fushan/trait.csv')

