### MERGE sweden DATA
rm(list = ls()); 
source("format.fun.R"); 
library(reshape);
dir.create("../../output/formatted/Sweden", recursive=TRUE,showWarnings=FALSE)

######################### READ DATA read individuals tree data
data.swe <- read.table("../../data/raw/Sweden/Swe_NFI_all.txt",header=T,stringsAsFactors=F,sep="\t")
#head(data.swe)
data.swe$tree.id <- apply(cbind(data.swe[["Year"]],data.swe[["TractNr"]],data.swe[["PlotNr"]],data.swe[["TreeNr"]]),1,paste,collapse="_")
data.swe$plot <- apply(cbind(data.swe[["Year"]],data.swe[["TractNr"]],data.swe[["PlotNr"]]),1,paste,collapse="_")
dim(data.swe)
#table(table(data.swe$TreeID))
#table(table(paste(data.swe$TractNr,data.swe$PlotNr,data.swe$TreeNr,data.swe$Year)))
#data.swe <- rbind(data.swe1, data.swe2, data.swe3); 
#rm(data.swe1, data.swe2, data.swe3)
#data.swe <- data.swe[order(data.swe$TreeID,data.swe$PlotInvent),] ## Shows the TreeID = "" first
#data.swe <- data.swe[order(paste(data.swe$TractNr,data.swe$PlotNr,data.swe$TreeNr)),] ## Shows the TreeID = "" first
## Format to desired form
data.swe$PlotInventID <- NULL; data.swe$TreeID <- NULL; 
data.swe2 <- data.swe[rep(1:nrow(data.swe),each=2),]
data.swe2$dbh1 <- as.vector(rbind(data.swe$dia_t1,data.swe$dia_t2))/10 ## conversion of dbh to cm
data.swe2$dbh2 <- as.vector(rbind(data.swe$dia_t2,data.swe$dia_t3))/10 ## conversion of dbh to cm
data.swe2$dia_t1 <- data.swe2$dia_t2 <- data.swe2$dia_t3 <- NULL
data.swe2$vol1 <- as.vector(rbind(data.swe$vol_t1,data.swe$vol_t2)) ## m3
data.swe2$vol2 <- as.vector(rbind(data.swe$vol_t2,data.swe$vol_t3)) ## m3
data.swe2$vol_t1 <- data.swe2$vol_t2 <- data.swe2$vol_t3 <- NULL
data.swe2$dryw1 <- as.vector(rbind(data.swe$DryW_t1,data.swe$DryW_t2)) ## kg
data.swe2$dryw2 <- as.vector(rbind(data.swe$DryW_t2,data.swe$DryW_t3)) ## kg
data.swe2$DryW_t1 <- data.swe2$DryW_t2 <- data.swe2$DryW_t3 <- NULL
data.swe2$census <- rep(1:2,nrow(data.swe2)/2) ## Using first census in the row
data.swe2$Diameter <- data.swe2$Volume <- data.swe2$BrhAge <- NULL
data.swe2$TractNr <- data.swe2$PlotNr <- data.swe2$TreeNr <- NULL
data.swe2$sp <- data.swe$TreeSpecies; data.swe$TreeSpecies <- NULL ## Species names are in the xlsx files if required
data.swe <- data.swe2
rm(data.swe2)
############## MASSAGE TRAIT DATA Compute maximum height per species plus sd from observed
############## height to add variables to the traits data base Because we have two heights,
############## then take the max of the two heights and then bootstrap
## Obtain heights in dataset - I assume that they use the volume of a cylinder V = pi*r^2*h
data.swe$ht1 <- data.swe$vol1/(pi*(0.5*data.swe$dbh1/100)^2) ## m
data.swe$ht2 <- data.swe$vol2/(pi*(0.5*data.swe$dbh2/100)^2) ## m
res.quant.boot <- t(sapply(levels(factor(data.swe[["sp"]])), FUN = f.quantile.boot, 
    R = 1000, x = log10(apply(data.swe[, c("ht1", "ht2")], 1, max, na.rm = T)), 
    fac = factor(data.swe[["sp"]])))
## create data base
data.max.height <- data.frame(code = rownames(res.quant.boot), Max.height.mean = res.quant.boot[, 
    1], Max.height.sd = res.quant.boot[, 2], Max.height.nobs = res.quant.boot[, 3], stringsAsFactors =FALSE)
rm(res.quant.boot)
write.csv(data.max.height,file='../../output/formatted/Sweden/max.height.csv') ### GEORGES NOT SURE WE SHOULD USE THAT

#################
##### change the projection of xy to wgs84 lat long.
#### change coordinates system of x y to be in lat long WGS84
library(sp)
library(dismo)
library(rgdal)
data.sp <- data.swe[, c("tree.id", "ecoord", "ncoord")]
coordinates(data.sp) <- c("ecoord", "ncoord")  # define x y
proj4string(data.sp) <- CRS("+init=epsg:3006")  # define projection system of our data ## EPSG:3006 check with Bertil
summary(data.sp)
detach(package:rgdal)
data.sp2 <- spTransform(data.sp, CRS("+init=epsg:4326"))  ## change projection in WGS84 lat lon
data.swe$Lon <- coordinates(data.sp2)[, "ecoord"]
data.swe$Lat <- coordinates(data.sp2)[, "ncoord"]

## ## ## ## plot on world map
## library(rworldmap)
## newmap <- getMap(resolution = 'coarse')
## ## # different resolutions available
## plot(newmap)
## points(data.sp2,cex=0.2,col='red')
rm(data.sp, data.sp2)




########################################## FORMAT INDIVIDUAL TREE DATA 
data.swe2 <- subset(data.swe, dbh1 >= 4 | is.na(dbh1)) ## Minimum DBH should be 40mm so remove anything below that
data.swe2 <- subset(data.swe2, dbh2 >= 4 | is.na(dbh2))
data.swe <- data.swe2
data.swe <- subset(data.swe,subset=!is.na(data.swe$dbh1))
dim(data.swe)
data.swe$cluster <- rep(NA, nrow(data.swe))  ## no plot cluster

## load species latin name
species.clean <- read.csv("../../data/raw/Sweden/species.list.csv",header=T,stringsAsFactors=F,sep=",")
data.swe$sp.name <- species.clean[match(data.swe$sp,species.clean$sp),"Latin_name"] ## GET LATIN NAME

data.swe$Year <- NULL; 
data.swe$year <- 5 ## number of year between measurement
data.swe$G <- 10*(data.swe$dbh2-data.swe$dbh1)/data.swe$year ## diameter growth in mm per year
data.swe$G[is.na(data.swe$dbh1)] <- NA
sum(data.swe$G < 0, na.rm = T) 
data.swe$D <- data.swe[["dbh1"]]; ## diameter in cm
data.swe$dead <- as.numeric(is.na(data.swe$dbh2)) ## dummy variable for dead tree 0 alive 1 dead
data.swe$htot <- rep(NA,nrow(data.swe)) ## height of tree in m
data.swe$obs.id <- apply(cbind(data.swe$tree.id,data.swe$census.id),1,paste,collapse="_")
data.swe$weights <-  rep(NA,nrow(data.swe)) ### DONE
data.swe$weights[data.swe$D<4] <- data.swe$PlotArea0139[data.swe$D<4]
data.swe$weights[data.swe$D>=4 & data.swe$D<10] <- data.swe$PlotArea4099[data.swe$D>=4 & data.swe$D<10]
data.swe$weights[data.swe$D>=10] <- data.swe$PlotArea100[data.swe$D>=10]
data.swe$weights <-   1/data.swe$weights

######################## ECOREGION
## One ecoregion for Sweden only?
data.swe$ecocode <- rep("A",nrow(data.swe))
## NEED TO ASK SWEDEN

######################## PERCENT DEAD
## variable percent dead
## compute numer of dead per plot to remove plot with disturbance
perc.dead <- tapply(data.swe[["dead"]],INDEX=data.swe[["plot"]],FUN=function.perc.dead2)
# ## VARIABLE TO SELECT PLOT WITH NOT BIG DISTURBANCE KEEP OFTHER VARIABLES IF AVAILABLE (disturbance record)
data.swe <- merge(data.swe,data.frame(plot=names(perc.dead),perc.dead=perc.dead), by = "plot", sort=FALSE)
############################################################## PLOT SELECTION FOR THE ANALYSIS
## Remove data with dead == 1
## table(data.swe$dead)
## data.swe <- subset(data.swe, dead == 0)

## vec.abio.var.names <- c("MAT", "MAP") ## TODO NO MAT MAP NEED TO LOAD FROM WORLDCLIM
vec.basic.var <- c("obs.id","tree.id", "sp", "sp.name","cluster","plot", "ecocode", "D", "G", "dead", 
    "year", "htot", "Lon", "Lat", "perc.dead","weights","census")
data.tree <- subset(data.swe, select = c(vec.basic.var))
write.csv(data.tree,file="../../output/formatted/Sweden/tree.csv")
